// TM1637 test
device = 18F26K22
clock = 64

#if (_device = 18F26K22)
config
    FOSC = INTIO67,     // Internal oscillator block
    PLLCFG = OFF,       // Oscillator used directly
    PRICLKEN = ON,      // Primary clock enabled
    FCMEN = OFF,        // Fail-Safe Clock Monitor disabled
    IESO = OFF,         // Oscillator Switchover mode disabled
    PWRTEN = ON,        // Power up timer enabled
    BOREN = SBORDIS,    // Brown-out Reset enabled in hardware only (SBOREN is disabled)
    BORV = 250,         // VBOR set to 2.50V nominal
    WDTEN = OFF,        // Watch dog timer is always disabled. SWDTEN has no effect.
    WDTPS = 32768,      // 1:32768
    CCP2MX = PORTC1,    // CCP2 input/output is multiplexed with RC1
    PBADEN = ON,        // PORTB<5:0> pins are configured as digital I/O on Reset
    CCP3MX = PORTB5,    // P3A/CCP3 input/output is multiplexed with RB5
    HFOFST = ON,        // HFINTOSC output and ready status are not delayed by the oscillator stable status
    T3CMX = PORTC0,     // T3CKI is on RC0
    P2BMX = PORTB5,     // P2B is on RB5
    MCLRE = EXTMCLR,    // MCLR pin enabled, RE3 input pin disabled
    STVREN = ON,        // Stack full/underflow will cause Reset
    LVP = OFF,          // Single-Supply ICSP disabled
    XINST = OFF,        // Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    DEBUG = OFF,        // Disabled
    CP0 = OFF,          // Block 0 (000800-003FFFh) not code-protected
    CP1 = OFF,          // Block 1 (004000-007FFFh) not code-protected
    CP2 = OFF,          // Block 2 (008000-00BFFFh) not code-protected
    CP3 = OFF,          // Block 3 (00C000-00FFFFh) not code-protected
    CPB = OFF,          // Boot block (000000-0007FFh) not code-protected
    CPD = OFF,          // Data EEPROM not code-protected
    WRT0 = OFF,         // Block 0 (000800-003FFFh) not write-protected
    WRT1 = OFF,         // Block 1 (004000-007FFFh) not write-protected
    WRT2 = OFF,         // Block 2 (008000-00BFFFh) not write-protected
    WRT3 = OFF,         // Block 3 (00C000-00FFFFh) not write-protected
    WRTC = OFF,         // Configuration registers (300000-3000FFh) not write-protected
    WRTB = OFF,         // Boot Block (000000-0007FFh) not write-protected
    WRTD = OFF,         // Data EEPROM not write-protected
    EBTR0 = OFF,        // Block 0 (000800-003FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF,        // Block 1 (004000-007FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF,        // Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF,        // Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks
    EBTRB = OFF         // Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
#endif

// intosc
include "intosc.bas"

// digital/analog mode
#option DIGITALIO_INIT = true   // automatically call setalldigital at startup
include "setdigitalio.bas"

// TM1637
#option TM1637_CLK = PORTB.0
#option TM1637_DIO = PORTB.1
' the datasheet app circuit shows 10K/100pF on the CLK and DIO lines,
' but the measured rise time was ~100us in OD mode
' with the caps removed the module was able to handle a bit_delay=1us with OD_CLK = true
'#option TM1637_BIT_DELAY = 1    // 100us clk (default)
'#option TM1637_OD_CLK = false  // true=OD mode(default), false=push-pull(no pullup reqd)
'#option TM1637_MAX_DIGITS = 4  // 4 digits (default), supports up to 6 digits
include "TM1637.bas"

// segment data for 4 chars
dim seg(4) as byte

const SEG_DONE() as byte = 
(
    SEG_B + SEG_C + SEG_D + SEG_E + SEG_G,           // d
    SEG_A + SEG_B + SEG_C + SEG_D + SEG_E + SEG_F,   // O
    SEG_C + SEG_E + SEG_G,                           // n
    SEG_A + SEG_D + SEG_E + SEG_F + SEG_G            // E
)

// TM1737 init event example
// enable PORTB pullups on CLK and DIO 
public event wpu_enable()
  const CLK_BIT = TM1637_CLK@   // get bit numbers from the options
  const DIO_BIT = TM1637_DIO@
  
  #if (TM1637_CLK = PORTB) and (TM1637_DIO = PORTB)
    wpub.bits(CLK_BIT) = 1
    wpub.bits(DIO_BIT) = 1
  #endif
end event

const SEG_INIT_DATA as byte = $FF   // all on
const MAX_BRIGHT as byte = 7        // 0 to 7

dim i, k as byte

main:
// this is redundant since option DIGITALIO_INIT=true already took care of it,
// so it's just here as a reminder
SetAllDigital()

// init pins (optional, done at startup... adding it here allows using a callback event)
TM1637.init(wpu_enable)

// all segments on
seg(0) = SEG_INIT_DATA
seg(1) = SEG_INIT_DATA
seg(2) = SEG_INIT_DATA
seg(3) = SEG_INIT_DATA
TM1637.setSegments(seg)

// turn on display (lowest brightness)
TM1637.displayCmd(0, 1)
delayms(1000)

// clear display
TM1637.clearDisplay()
delayms(500)

// Run through all the dots
for i = 0 to 3
    k = 1
    repeat
        seg(i) = k    
        TM1637.setSegments(seg)
        k = k << 1
        delayms(100)
    until (k = $80)
next
delayms(1000)

// on/off test
seg(0) = SEG_INIT_DATA
seg(1) = SEG_INIT_DATA
seg(2) = SEG_INIT_DATA
seg(3) = SEG_INIT_DATA
TM1637.setSegments(seg)
delayms(500)
for i = 1 to 3
    TM1637.displayCmd(MAX_BRIGHT, 0)
    delayms(500)
    TM1637.displayCmd(MAX_BRIGHT, 1)
    delayms(500)
next

// brightness test
for i = 0 to MAX_BRIGHT
    seg(0) = TM1637.digitToSegment(i)
    seg(1) = TM1637.digitToSegment(i)
    seg(2) = TM1637.digitToSegment(i)
    seg(3) = TM1637.digitToSegment(i)
    TM1637.setSegments(seg)
    TM1637.displayCmd(i, 1)
    delayms(500)
next

// sample clock displays
TM1637.DisplayClock("1234")        // no colon
delayms(500)
TM1637.DisplayClock("12:34")
delayms(500)
TM1637.DisplayClock("--:--")
delayms(500)
TM1637.DisplayClock("00:00")
delayms(500)

// Done!
seg = SEG_DONE
TM1637.setSegments(seg)
TM1637.displayCmd(MAX_BRIGHT, 1)

while (true)
end while
